     
import sys, time 
from math import sin,cos,sqrt,pi

import animators
from renderer import *
import lib.texture_manager

world_object_list = []
t = 0
period = 30        # num of ms per anim tick
logic_tick_period = 10 #num of anim ticks per logic tick
animators.set_logic_tick_period (logic_tick_period)

class command:
    robot = 0
    def set_robot (self,robot):
        self.robot = robot
    def execute(self):
        pass
    
class forward_command (command):
    count = 0
    def __init__ (self,count):
        self.count = count
    def execute (self):
        if (self.count > 0):
            self.robot.set_pos ([p+ d for p,d in zip(self.robot.get_pos(),self.robot.get_dir())])
            self.robot.moving=True
        self.count -= 1
        return self.count <= 0
        
class left_command (command):
    def execute (self):
        current_dir = self.robot.get_dir()
        self.robot.set_dir ([-current_dir[1],current_dir[0]])
        return True
        
class right_command (command):
    def execute (self):
        current_dir = self.robot.get_dir()
        self.robot.set_dir ([current_dir[1],-current_dir[0]])
        return True
    

def char_to_command (char):
    if (char == '5'):
        return forward_command (5)
    elif (char == '1'):
        return forward_command (1)
    elif (char == 'l'):
        return left_command ()
    elif (char == 'r'):
        return right_command()
    else :
        return forward_command (0) #use this as an empty default



class object:
    pos  = [0,0] 
    old_pos = [0,0]
    dir = [1,0]
    moving = False
    anim = 0
    def __init__ (self):
        #print "object con"
        self.anim = animators.moving_box_with_blink(self)
    def get_colour (self):
        return [1,1,1]
        
    def set_pos (self,pos):
        self.old_pos = self.pos
        self.pos = pos
    def get_pos (self):
        return self.pos
        
    def set_dir (self,dir):
        self.dir = dir
    def get_dir (self):
        return self.dir
    

    
    
class robot (object):
    command_string = ""
    commands = []
    current_command = 0
    def get_colour (self):
        return [0.3,0.3,1]
        
        
    def __init__ (self, p_pos):
        self.pos = p_pos
        self.anim = animators.moving_box_with_blink(self)
        
    def set_commands (self , str):
        self.command_string = str
        self.current_command = 0
        self.commands = []
        for char in str:
            com = char_to_command (char)
            self.commands.append (com)
            com.set_robot (self)
    def execute (self):
        #return moving variable to default state
        self.moving = False
        if (len(self.commands)>self.current_command):
            if (self.commands[self.current_command].execute() == True):
                #its finished this command
                self.current_command += 1
                #if (len(self.commands)>self.current_command):
                 #   print self.commands[self.current_command]


def draw ():
    render_start ()
    for obj in world_object_list:
        obj.anim.draw(t%logic_tick_period,t)
    render_end()

    
def init_world ():
    global world_object_list
    #create robot
    y=2
    if (len(sys.argv)>1):
        for arg in sys.argv[1:]:
            r = robot([2,y])
            world_object_list.append (r)
            r.set_commands (arg)
            y+=1
    else:
        r = robot([2,2])
        world_object_list.append (r)
        #print (len(world_object_list))
        r.set_commands ("5l5r1r1r51l1l5r5l1l5")
        r = robot([1,2])
        world_object_list.append (r)
        #print (len(world_object_list))
        r.set_commands ("1l1r5l5l1l5l1rl1l5")
 
def tick (value):
    global t
    glutTimerFunc (period,tick,0)
    t += 1
    if (t % logic_tick_period == 0):
        for obj in world_object_list:
            obj.execute()
    glutPostRedisplay ()
def main ():
    init()
    
    lib.texture_manager.loadTextures (["metal2"])
    glutTimerFunc (period,tick,0)
    glutDisplayFunc(draw)

    
    init_world()
    #print "starting"
    glutMainLoop()
    
 
if __name__ == '__main__':
    main ()
