"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
#Animators
import random, sys
from rendering_utils import renderBox, renderExplosion
import rendering_utils
import renderer
import texture_manager
import math

class animator:
    parent = 0

    def draw (self, phase, tick):
        pass
    def find_apparent_pos (self,phase):
        if (self.parent.moving):
            old_pos = self.parent.old_pos
            self.apparent_pos = [p+d*float(phase) for p,d in zip (old_pos,self.parent.get_dir())]+[0]
        else:
            self.apparent_pos = self.parent.get_pos()+[0]
    

class null_animator(animator):
    apparent_pos = [0,0,0]
    def __init__(self, parent):
        self.parent = parent


class moving_box_animator (animator):
    apparent_pos = [0,0,0]
    def __init__ (self,parent):
        self.parent = parent

    def draw (self,phase,tick): #phase is 0 to (logic_tick_period-1) and indicates how far through the current anim cycle we are
        self.find_apparent_pos(phase)
        self.apparent_pos[2]=0.5  #centre or box is raised so box is on ground
        #draw_coloured_square (self.apparent_pos,self.parent.get_colour())
        #draw_box(self.apparent_pos,self.parent.get_colour())
        renderer.bind_texture ("metal2")
        renderBox(self.apparent_pos, [1,1,1])

class moving_box_with_blink (moving_box_animator):
    random_offset = 0
    def __init__ (self,parent):
        self.parent = parent
        self.random_offset = random.randint(0,1000)

    def draw (self,phase,tick):
        moving_box_animator.draw(self,phase,tick) #let the base draw the box
        
        if ((tick+self.random_offset) % 2000 < 1000): #tick is just tick time and unrelated to phase
            color = [1,0,0]
        else:
            color = [1,1,1]
        renderer.set_color (color)
        blink = self.apparent_pos
        blink[2]+=0.6
        renderBox(blink, [0.2,0.2,0.2])

class particle_data:
    def __init__(self):
        self.position = [0, 0, 0]
        self.speed = [0, 0, 0]
        self.acceleration = [0, 0, -1]
        self.colour = [1, 0.17, 0.011]  

class debris_data:
    def __init__(self):
        self.position = [0, 0, 0]
        self.speed = [0,0,0]
        self.acceleration = [0, 0, -1]
        self.orientation = [0,0,0]
        self.orientationSpeed = [0,0,0]
        self.colour = [1,0.17,0.011]
        self.scale = [0,0,0]

class explosion_animator(animator):
    def __init__(self, parent):
        self.parent = parent
        self.particles = []
        self.numRenders = 0
        for i in range(100):
            particle = particle_data()
            particle.speed[0] = (2 * random.random()) - 1.0
            particle.speed[1] = (2 * random.random()) - 1.0
            particle.speed[2] = (2 * random.random()) - 1.0
            self.particles.append(particle)

        self.debris = []
        for i in range(70):
            one_debris = debris_data()
            one_debris.scale[0] = (2 * random.random()) - 1.0
            one_debris.scale[1] = (2 * random.random()) - 1.0
            one_debris.scale[2] = (2 * random.random()) - 1.0
            one_debris.speed[0] = (2 * random.random()) - 1.0
            one_debris.speed[1] = (2 * random.random()) - 1.0
            one_debris.speed[2] = (2 * random.random()) - 1.0
            one_debris.orientationSpeed[0] = (2 * random.random()) - 1.0
            one_debris.orientationSpeed[1] = (2 * random.random()) - 1.0
            one_debris.orientationSpeed[2] = (2 * random.random()) - 1.0
            self.debris.append(one_debris)
            

    def draw(self, phase, tick):
        for particle in self.particles:
            particle.position[0] += particle.speed[0] * 0.02
            particle.position[1] += particle.speed[1] * 0.02
            particle.position[2] += particle.speed[2] * 0.02

            particle.speed[2] += particle.acceleration[2] * 0.02
            if particle.position[2] < 0:
                self.particles.remove(particle)

        for debri in self.debris:
            debri.position[0] += debri.speed[0] * 0.01
            debri.position[1] += debri.speed[1] * 0.01
            debri.position[2] += debri.speed[2] * 0.01

            debri.speed[2] += debri.acceleration[2] * 0.01

            debri.orientation[0] += debri.orientationSpeed[0] * 0.10
            debri.orientation[1] += debri.orientationSpeed[1] * 0.10
            debri.orientation[2] += debri.orientationSpeed[2] * 0.10

        renderer.push_position ()
        self.find_apparent_pos(phase)
        ap = self.apparent_pos
        renderer.translate(*ap)
        renderExplosion(self.particles, self.debris)
        renderer.pop_position()
        self.numRenders += 1
        if len(self.particles) == 0:
            self.parent.level.renderableList.remove(self.parent)
 
class robot_animator (animator):
    random_offset = 0
    leg_offset = 0.1
    leg_size = [0.1,0.15,0.5]
    body_size = [0.5,0.3,0.6]
    arm_size = [0.2,0.1,0.1]
    head_size = [0.2,0.2,0.2]
    arm_height = 0.5 #distance from bottom of body section
    
    def __init__ (self,parent):
        self.parent = parent
        self.random_offset = random.randint(0,1000)

    def render_leg (self,theta):
        renderer.push_position ()
        renderer.translate (0,0,self.leg_size[2])
        renderer.rotate (theta,1,0,0)
        renderBox ([0,0,-self.leg_size[2]/2.0],self.leg_size)
        renderer.pop_position ()
        
    def render_arm (self,theta):
        renderer.push_position ()
        renderer.rotate (theta,0,1,0)
        renderBox ([self.arm_size[0]/2,0,0],self.arm_size)
        renderer.pop_position ()
        
    def draw (self,phase,tick):
        tick = tick + self.random_offset
        theta = math.sin(tick/300.0)*30
        if (self.parent.legs_moving):
            leg_theta = theta
        else:
            leg_theta = 0
        self.find_apparent_pos(phase)
        ap = self.apparent_pos # for less typing
        if not self.parent.created:
            renderer.enable_blending()
            renderer.set_blend_color (self.parent.get_color(), 0.81)
        else:
            renderer.set_color (self.parent.get_color())
        renderer.bind_texture ("metal2")
        renderer.push_position ()
        renderer.translate (*ap)
        if (self.parent.dir[0] != 0):
            renderer.rotate (90,0,0,1)
        renderer.push_position ()
        
        renderer.translate (self.leg_offset,0,0)
        self.render_leg (leg_theta)
        renderer.translate (-2*self.leg_offset,0,0)
        self.render_leg (-leg_theta)
        renderer.pop_position()
        renderer.push_position ()
        renderer.translate (0,0,self.leg_size[2]+self.body_size[2]/2.0)
        #render body
        renderBox ([0,0,0],self.body_size)
        renderer.push_position ()
        
        renderer.translate (self.body_size[0]/2.0,0,self.arm_height-self.body_size[2]/2.0)
        
        self.render_arm(theta)
        renderer.translate (-self.body_size[0],0,0)
        renderer.rotate (180,0,0,1)
        self.render_arm(-theta)
        renderer.pop_position()
        renderer.translate (0,0,self.head_size[2]/2.0+self.body_size[2]/2.0)
        #render head
        renderBox ([0,0,0],self.head_size)
        
        renderer.pop_position()
        renderer.pop_position()
        if not self.parent.created:
            renderer.disable_blending()
        
class sinking_animator(robot_animator):
    def find_apparent_pos (self,phase):
        self.apparent_pos = self.parent.get_pos()
    def draw(self, phase, tick):
        self.parent.pos[2] -= 0.01
        if self.parent.pos[2] < -1:
            self.parent.level.remove_robot(self.parent)
            
            
        robot_animator.draw(self, phase, tick)

class tile_animator (animator):
    def __init__ (self,parent,texture_name,pos):
        self.parent = parent
        self.texture = texture_manager.getTexture(texture_name)
        self.pos = pos

    def draw(self, phase, tick):
        renderer.set_color ([1,1,1])
        renderer.bind_texture_obj (self.texture)
        rendering_utils.renderQuad (self.pos[0]-0.5,self.pos[1]-0.5,self.pos[0]+0.5,self.pos[1]+0.5)
        
class switch_animator (animator):
    def __init__ (self,parent,pos,color):
        self.color = color
        self.parent = parent
        self.floor_texture = texture_manager.getTexture("floor1")
        self.button_texture = texture_manager.getTexture("metal2")
        self.pos = pos

    def draw(self, phase, tick):
        renderer.set_color([1,1,1])
        renderer.bind_texture_obj (self.floor_texture)
        rendering_utils.renderQuad (self.pos[0]-0.5,self.pos[1]-0.5,self.pos[0]+0.5,self.pos[1]+0.5)
        renderer.set_color (self.color)
        renderer.bind_texture_obj (self.button_texture)
        rendering_utils.renderBox (self.pos,[0.3,0.3,0.1])
        
class token_animator (animator):
    def __init__ (self,parent,texture_name,token_texture_name,pos):
        self.parent = parent
        self.texture = texture_manager.getTexture(texture_name)
        self.token_texture = texture_manager.getTexture(token_texture_name)
        self.pos = pos

    def draw(self, phase, tick):
        renderer.set_color ([1,1,1])
        renderer.bind_texture_obj (self.texture)
        rendering_utils.renderQuad (self.pos[0]-0.5,self.pos[1]-0.5,self.pos[0]+0.5,self.pos[1]+0.5)
        
        renderer.push_position ()
        renderer.bind_texture_obj (self.token_texture)
        renderer.translate (self.pos[0],self.pos[1],0.5)
        renderer.rotate (int (tick/6) %360,0,0,1)
        renderer.rotate (90,1,0,0)
        rendering_utils.renderQuad (-0.4,-0.4,0.4,0.4)
        renderer.rotate (180,0,1,0)
        rendering_utils.renderQuad (-0.4,-0.4,0.4,0.4)
        
        renderer.pop_position()
        
        
        
        
class spawn_animator (animator):
    fl = [-0.5,-0.5,0]
    fr = [0.5,-0.5,0]
    bl = [-0.5,0.5,0]
    br = [0.5,0.5,0]
    blu = [-0.5,0.5,0.1]
    bru = [0.5,0.5,0.1]
    back_bit = [0,0.9,0]
    
    def __init__ (self,parent,texture_name,base_texture_name,pos):
        
        
        self.parent = parent
        self.texture = texture_manager.getTexture(texture_name)
        self.base_texture = texture_manager.getTexture(base_texture_name)
        self.pos = pos

    def draw(self, phase, tick):
        renderer.push_position ()
        renderer.translate(self.pos[0],self.pos[1],0)
        
        renderer.set_color ([1,1,1])
        renderer.bind_texture_obj (self.base_texture)
        rendering_utils.renderQuad (-0.5,-0.5,0.5,0.5)
        renderer.bind_texture_obj (self.texture)
        if (self.parent.dir==[1,0]):
            renderer.rotate(90,0,0,1)
        elif (self.parent.dir==[0,1]):
            renderer.rotate(180,0,0,1)
        elif (self.parent.dir==[-1,0]):
            renderer.rotate(270,0,0,1)
        
        if (self.parent.activated):
            renderer.set_color ([0,1,0])
        else:
            renderer.set_color ([1,1,1])
        
        rendering_utils.renderSpawn ([0,0,0])
        #if (not self.parent.activated):
            #rendering_utils.renderBox ([0,-0.4,0.1],[0.7,.2,0.2])
        if (self.parent.selected):
            
            renderer.rotate(tick/20.0,0,0,1)
            rendering_utils.renderPointer([0,0,2 + 0.3*math.sin(tick/1000.0)],[0.5,0.5,0.7],[1,1,0])
        
        renderer.pop_position ()
        #rendering_utils.renderQuad (self.pos[0]-0.5,self.pos[1]-0.5,self.pos[0]+0.5,self.pos[1]+0.5)
        
class wall_animator (animator):
    def __init__ (self,parent,texture_name,pos):
        self.parent = parent
        self.texture = texture_manager.getTexture(texture_name)
        self.pos = pos
        self.sides = [True,False,False,False]
    def draw(self, phase, tick):
        renderer.set_color ([1,1,1])
        renderer.bind_texture_obj (self.texture)
        #rendering_utils.renderQuad (self.pos[0]-0.5,self.pos[1]-0.5,self.pos[0]+0.5,self.pos[1]+0.5)
        
        rendering_utils.renderPartialBox(self.pos,(1,1,1,1), self.sides)

        
class gate_animator (animator):
    def __init__ (self,parent,texture_name,pos,color):
        self.parent = parent
        self.texture = texture_manager.getTexture(texture_name)
        self.pos = pos
        self.color = color

    def draw(self, phase, tick):
        renderer.set_color (self.color)
        renderer.bind_texture_obj (self.texture)
        #rendering_utils.renderQuad (self.pos[0]-0.5,self.pos[1]-0.5,self.pos[0]+0.5,self.pos[1]+0.5)
        rendering_utils.renderQuad (self.pos[0]-0.5,self.pos[1]-0.5,self.pos[0]+0.5,self.pos[1]+0.5,height =0.01)
        if (self.parent.old_state == True and self.parent.is_open):
            return
        
 
        if (self.parent.is_open):
            portion = 1.0 - phase
        else:
            if (self.parent.old_state == False):
                portion = 1.0
            else:
                portion = phase
        
        renderer.push_position()
        renderer.translate(self.pos[0],self.pos[1],2-(1-portion)*2)
        renderer.rotate(180,1,0,0)
        rendering_utils.renderPointer ([0,0,0],[1,1,2],[a*0.4 for a in self.color])
        renderer.pop_position()
        #rendering_utils.renderBox([self.pos[0],self.pos[1],self.pos[2]+height/2], (1, 1, height, 1))        

