"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
import math
from rendering_utils import angle ,subtract

class dragger:
    world_pos_getter = 0
    get_camera_func = 0
    set_camera_func = 0
    
    dragging = False
    spinning = False
    dragging_world_point = []
    start_mouse_pos = []
    start_camera_pos =[]
    
    def set_screen_size (self,size):
        self.screen_size = size
        self.mid_screen = [self.screen_size[0]/2,self.screen_size[1]/2]
    def camera_move_delta (self,world_pos_now,desired_world_pos):
        """returns the required position of the camera for what pointed at world_pos_now to point
        at desired_world_pos"""
        return [a-(b-c) for a,b,c in zip(self.get_camera_func(),world_pos_now,desired_world_pos)]
    def mouse_down (self,pos,buttons):
        if (self.spinning or self.dragging):
            return
        self.start_mouse_pos = pos
        
        if (buttons == 3):
            self.dragging_world_point = self.world_pos_getter (self.mid_screen)
            self.spinning = True
            self.theta = self.get_camera_angle_func ( ) /(180/math.pi)
            self.start_angle = self.get_camera_angle_func()
        else:    
            self.dragging_world_point = self.world_pos_getter (pos)
            self.dragging = True
            self.start_camera_pos = self.get_camera_func()
        
    def mouse_up (self,pos):
        
        self.dragging = False
        self.spinning = False
        
    def mouse_move (self,pos):    
        
        if (self.dragging):
            pos_now = self.world_pos_getter (pos)
            self.set_camera_func (self.camera_move_delta(pos_now,self.dragging_world_point))
        if (self.spinning):
            theta = angle (subtract (pos , self.mid_screen),subtract (self.start_mouse_pos,self.mid_screen))
            self.theta += theta
            self.set_camera_angle_func (self.theta * 180/math.pi)
            self.centre_on_square (self.mid_screen,self.dragging_world_point)
            self.start_mouse_pos = pos
            
    
    
    def wheel_down (self):
        self.wheel_up(delta= - 1)
       
    def wheel_up (self, delta = 1):
        pos_now = self.world_pos_getter (self.mid_screen)
        cam_pos = self.get_camera_func()
        vector = [a-b for a,b in zip(cam_pos,pos_now)]
        vector_length = math.sqrt (vector[0]*vector[0] + vector[1]*vector[1]+ vector[2]*vector[2])
        
        self.set_camera_func ([a - delta*b/vector_length for a,b in zip (cam_pos,vector)])
            
    def set_world_pos_getter (self,world_pos_getter):
        self.world_pos_getter = world_pos_getter
        
    def set_camera_funcs (self,get_camera_pos,set_camera_pos,get_camera_angle_func,set_camera_angle_func,camera_distance_change):
        self.get_camera_func = get_camera_pos
        self.set_camera_func = set_camera_pos
        self.get_camera_angle_func = get_camera_angle_func
        self.set_camera_angle_func = set_camera_angle_func
        self.camera_distance_change = camera_distance_change
        
    def centre_on_square (self,screen_pos,pos):
        pos_now = self.world_pos_getter (screen_pos)
        self.set_camera_func (self.camera_move_delta(pos_now,[pos[0],pos[1],0]))
        
