"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
import pygame
from pygame.locals import *

import menus

ticks_per_execute = 500

show_fps = True
fps_period = 5000.0

class Engine:
    def __init__(self, renderer, size, fullscreen):
        renderer.WIDTH = size[0]
        renderer.HEIGHT = size[1]
        self.renderer = renderer
        self.renderer.init(size,fullscreen)
        self.levels = []
        self.currentLevel = 0
        self.running = False
        self.levelConfig = {}
        self.total_time = 0
        
        self.playing = False
    def start(self):
        lastTime = pygame.time.get_ticks()
        accumulated = 0
        total_running_time = 0
        frames = 0
        last_fps_output = 0
        done = False

        while not done:
            # handle logic
            time = pygame.time.get_ticks()
            self.time = time
            if (self.running):
                accumulated += time - lastTime
                total_running_time += time - lastTime
            self.total_time += time - lastTime
            lastTime = time
            while(accumulated >= ticks_per_execute):
                accumulated -= ticks_per_execute
                self.execute()

            # Handle events
            event = pygame.event.poll()
            if (event.type == MOUSEBUTTONDOWN  and event.button < 4): # < 4 is not a silly wheel event
                self.mouse_down(event.pos,event.button)
            if (event.type == MOUSEMOTION  ):
                self.mouse_move(event.pos)
            if (event.type == MOUSEBUTTONUP  ):
                self.mouse_up(event.pos)
            if event.type == QUIT:
                done = True 
            if (event.type == MOUSEBUTTONDOWN and event.button == 4):
                self.wheel_up()
            if (event.type == MOUSEBUTTONDOWN and event.button == 5):
                self.wheel_down()
            if (event.type == KEYDOWN and event.key == K_ESCAPE):
                self.in_menu = True
                if (self.menu == 0):
                    self.menu = menus.MainMenu (self)
                
            # Render the level
            self.renderer.render()
            self.render(total_running_time,float(accumulated%ticks_per_execute)/ticks_per_execute)
            pygame.display.flip()
            frames += 1
            if (last_fps_output < (time - fps_period )):
                last_fps_output = time
                #print "FPS = " , frames*1000/fps_period
                frames = 0

    def addLevel(self, level):
        self.levels.append(level)
        self.level = self.levels[self.currentLevel]

    def resetLevel(self):
        self.load_level("level" + str(self.currentLevel))


    def nextLevel(self): 
        #print "nextLevel"
        self.currentLevel += 1
        #print self.currentLevel, self.max_level
        if self.currentLevel == self.max_level:
            return False
            
        #print "loading level:",str(self.currentLevel)
        self.load_level("level" + str(self.currentLevel))
        
        return True
