"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
import rendering_utils
from animators import moving_box_with_blink
from tokens import char_to_command
from constants import *
import animators

class GameObject:
    pos  = [0,0]
    old_pos = [0,0]
    dir = [1,0]
    moving = False
    anim = 0
    def __init__ (self, level):
        #print "object con"
        self.anim = moving_box_with_blink(self)
        self.level = level
        
    def set_pos (self,pos):
        self.old_pos = self.pos
        self.pos = pos
    def get_pos (self):
        return self.pos

    def set_dir (self,dir):
        self.dir = dir
    def get_dir (self):
        return self.dir
    def execute (self):
        pass
    def complete (self):
        pass
    def get_map_element (self):
        return self.level.getMapElement(self.pos)

def getCommandFromSizeIndex (list,index):
    counter = 0
    for i,command in enumerate(list):
        if((counter+command.length-1) >= index):
            return list[i]
            
        counter += command.length
        
class MemoryBank:
    def __init__(self, memorySize):
        #print "start"
        self.memory = []
        self.length = 0
        self.maxLength = memorySize
        #print "end"
    
    def addCommand(self, command):
        if((self.length + command.length) > self.maxLength):
            return False
        self.memory.append(command)
        self.length += command.length
       
    
    def deleteCommand(self, index):
        #print "index:",index
        counter = 0
        for i,command in enumerate(self.memory):
            if((counter+command.length-1) >= index):
                self.length -= self.memory[i].length
                del self.memory[i]
                break
            counter += command.length

    def __len__(self):
        return len(self.memory) 

    def __getitem__(self,index):
        return self.memory[index]
        
class SinkingRobot(GameObject):
    def __init__(self, level, p_pos):
        GameObject.__init__(self, level)
        self.pos = p_pos + [0]
        self.anim = animators.sinking_animator(self)
        self.pre_anim = animators.null_animator(self)
    def execute(self):
        pass
        #self.pos[2] -= 0.1
        #if self.pos[2] < -1:
        #    self.level.remove_robot(self)
    def legs_moving(self):
        return False
    def get_color (self):
        if self.created:
	    return [0.3,0.3,1]
	else:
            return [0.5,0.5,0.5]
         
class Robot (GameObject):
    
    def get_color (self):
        if self.created:
            return [0.3,0.3,1]
        else:
            return [0.5,0.5,0.5]
    def __init__ (self, level, p_pos):
        GameObject.__init__(self, level)
        self.pos = p_pos
        self.spawn = None
        self.anim = animators.robot_animator(self)
        self.pre_anim = animators.null_animator(self)
        self.command_string = ""
        self.commands = MemoryBank(MEMORY_SIZE)
        self.current_command = 0
        self.legs_moving = False
        self.created = False
        
    def set_commands (self , str):
        self.command_string = str
        self.current_command = 0
        self.commands = MemoryBank(MEMORY_SIZE)
        for char in str:
            com = char_to_command(char)
            self.commands.addCommand(com)
            com.set_robot (self)
    def execute (self):
        #return moving variable to default state
        self.moving = False
        self.legs_moving = False
        if (len(self.commands)>self.current_command):
            if (self.commands[self.current_command].execute(self.level) == True):
                #its finished this command
                self.current_command += 1
                #if (len(self.commands)>self.current_command):
                #   print self.commands[self.current_command]
            elif not self.commands[self.current_command].isValid:
                self.current_command = len(self.commands)
        else:
            if self.level.engine.tutorial:
                self.level.remove_robot(self)
                self.level.renderableList.append(Explosion(self.level, self.pos))
            
                #self.level.doTutorialStep()
            elif self.created:
                self.level.remove_robot(self)
                self.level.renderableList.append(Explosion(self.level, self.pos))
        
        if not self.created:
            self.moving = False
            return
        
        #Check for nefarious villians
        if self.level.getMapElement(self.pos).enemies:
            self.level.remove_robot(self)
            self.level.renderableList.append(Explosion(self.level, self.pos))

        #Check if we just hit a deadly lava pool
        if self.level.getMapElement(self.pos).is_deadly():
            self.level.remove_robot(self, False)
            robot = SinkingRobot(self.level, self.pos)
            robot.created = True
            self.level.renderableList.append(Explosion(self.level, self.pos))
            self.level.renderableList.append(robot)
            self.level.executables.append(robot)
            
            

        #check current position
        if (self.level.getMapElement(self.pos).is_goal()):
            if not self.level.engine.tutorial:
                self.level.complete = True
        self.level.getMapElement(self.pos).stand_on_action()
    def push(self):
        if (self.created):
            self.get_map_element().push()
    def hack(self):
        if (self.created):
            self.get_map_element().hack()
    def moveToSpawn(self):
        #print "move to spawn"
        self.current_command = 0
        self.pos = self.spawn.pos
        self.dir = self.spawn.dir
        for command in self.commands:
            command.reset()
            self.current_command = 0
            self.pos = self.spawn.pos
            self.dir = self.spawn.dir
            for command in self.commands:
                command.reset()
    
        
class Enemy(Robot):
    def get_color (self):
        return [1,0,0]
        
    def __init__(self, level, p_pos):
        Robot.__init__(self, level, p_pos)
    def execute (self):
        self.moving = False
        self.legs_moving = False 
        self.level.getMapElement(self.pos).enemies-=1
        if len(self.commands)>self.current_command:
            if self.commands[self.current_command].execute(self.level) == True:
               self.current_command += 1 
            elif not self.commands[self.current_command].isValid:
                self.current_command = len(self.commands)
        self.level.getMapElement(self.pos).enemies+=1 

class Explosion(GameObject):
    def __init__(self, level, pos):
        GameObject.__init__(self, level)
        self.pos = pos
        self.anim = animators.explosion_animator(self)
        self.pre_anim = animators.null_animator(self)
