"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
#External imports
import os, os.path, re
import ConfigParser as CP

# Internal library imports
from level import Level
import tile_objects as TO

class SpawnFactory:
    def __init__(self, type):
        self.type = type 
    def __call__(self, pos):
        return TO.Spawn(pos, self.type)

MapTiles = {
    "#": TO.Wall,
    "<": SpawnFactory([-1,0]),
    ">": SpawnFactory([1,0]),
    "^": SpawnFactory([0,1]),
    "v": SpawnFactory([0,-1]),
    "%": TO.Lava,
    "*": TO.TokenLocation,
    " ": TO.Tile,
    #"a": TO.Switch,
    #"A": TO.Gate,
}

CONSTANTS = """
FILE_NAME
""".strip().split()

globs = globals()
for constant in CONSTANTS:
    globs[constant] = constant

def load_config(engine, configFile, path=os.path.join("data", "levels")):
    config_parser = CP.ConfigParser()
    read_files = config_parser.read(os.path.join(path, configFile))
    if not read_files:
        return
    for section in config_parser.sections():
        engine.levelConfig[section] = config_parser.get(section, FILE_NAME)
        
    return len(config_parser.sections())+1

def load_level(engine, name, path=os.path.join("data", "levels")):
    #print "utils load_lvel"
    #print engine.levelConfig
    #print name
    if engine.levelConfig.has_key(name):
        #print "matched name"
        filename = engine.levelConfig[name]
        level = Level(engine)
        if load_map_data(os.path.join(path, filename), level):
            return level
    else:
        raise Exception("No file %s" % name)
        
current_level = 0

gate_colors  = {'A' : [1,0,0], 
                'B' : [0,1,0], 
                'C' : [0,0,1], 
                'D' : [1,1,0], 
                'E' : [0,1,1], 
                'F' : [1,0,1], 
                'G' : [1,1,1]}
def load_map_data(path, level):
    global current_level
    current_level    = level
    if not os.path.exists(path):
        return False
    map = open(path, "r")
    name = map.readline()
    w, h = [int(size.strip()) for size in map.readline().split(",")]
    level.setSize(w, h)
    
    gates = {}
    switches = {}
    for line_num in range(h):
        x = 0
        y = (h-1)-line_num
        line = map.readline()
        if len(line.rstrip('\n')) > level.width or y < 0:
            raise Exception("Map data is trying to draw outside level bounds.")
        
        for e in line:
            if MapTiles.has_key(e):
                map_tile = MapTiles[e]([x, y, 0.5])
                map_tile.do_setup(level)
                level.setMapElement(map_tile, (x, y))
            else:
                if re.search("[A-Z]", e): # a gate
                    if not gates.has_key(e):
                        gates[e] = []
                    new_gate = TO.Gate((x, y, 0.0),gate_colors[e])
                    gates[e].append (new_gate)
                    level.setMapElement(new_gate, (x, y))
                elif re.search("[a-z]", e): # a switch
                    if not switches.has_key(e):
                        switches[e] = []
                    new_switch = TO.Switch((x, y, 0.0),gate_colors[e.upper()])
                    switches[e].append (new_switch)
                    level.setMapElement(new_switch, (x, y))
                    
            x += 1
        level.compile()
        
    
        
    #connect up gates and switches
    for e in switches:
        E = e.upper()
        for s in switches[e]:
            if (gates.has_key(E)):
                for g in gates[E]:
                    s.add_gate(g)
    
    for line in map:
        eval (line)
    
    level.finalise()
    
    return True
        

#Functions to be used from level files:
def EnableMapElement (x,y):
    current_level.getMapElement ([x,y]).enable()

def setTitle(title):
    current_level.title = title
def setText(text):
    current_level.text = text

def tutorialRobot(commandstr, index = 0):
    current_level.tutorial_steps.append(["robot",(commandstr,index)])

def tutorialDisable(index):
    current_level.tutorial_steps.append(["disable",index])

def tutorialText(text):
    current_level.engine.tutorial_text = text

