"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
import renderer
import rendering_utils

import sys

item_width = 450
item_height = 60

class MenuItem:
    def __init__(self,name,pos,tex):
        self.pos = pos
        self.name = name
        self.tex = tex
        self.action =0
        self.is_down = False
    def render (self):
        renderer.set_color ([1,1,1])
        renderer.bind_texture (self.tex)
        rendering_utils.renderQuad (
            self.pos[0]-item_width/5.0,
            self.pos[1]-item_height/2.0,
            self.pos[0]+item_width/5.0,
            self.pos[1]+item_height/2.0
        )
        
    def is_inside(self, pos):
        #print pos , self.pos[0]-item_width/2.0,self.pos[0]+item_width/2.0,self.pos[1]+item_height/2.0
        y_pos = (renderer.HEIGHT - 1) -pos[1]
        if (pos[0] > self.pos[0]-item_width/5.0 and
            pos[0] < self.pos[0]+item_width/5.0 and
            y_pos > self.pos[1]-item_height/2.0 and
            y_pos < self.pos[1]+item_height/2.0):
            
            return True
        return False
            
            

    def mouse_move (self,pos):
        pass
    def mouse_up (self,pos):
        pass
    def mouse_down (self,pos):
        if self.is_inside(pos):            
            return True
            
class Menu:
    def __init__ (self):
        self.items = []
        self.main_texture = 0
        self.area = 0
        self.down_item = None
    def render (self):
        renderer.disable_lighting ()
        renderer.start_pixel_map_mode()
        renderer.set_color ([1,1,1])
        renderer.bind_texture (self.main_texture)
        rendering_utils.renderQuad (*self.area)
        renderer.bind_texture ("floor1")
        for i in self.items:
            i.render()
        renderer.end_pixel_map_mode()
        renderer.enable_lighting ()
    def mouse_move (self,pos):
        for i in self.items:
            i.mouse_move(pos)
    def mouse_down (self,pos,buttons):
        for i in self.items:
            if i.mouse_down(pos):
                self.down_item = i
                break
    def mouse_up (self,pos):
        #print "t"
        #print self.items
        #for i in self.items:
        #    i.mouse_up(pos)
        if self.down_item:
            if self.down_item.is_inside(pos):
                if self.down_item.action:
                    self.down_item.action()
    

class OkDialog(Menu):
    def __init__ (self,title,text,ok_func):
        self.ok_func = ok_func
        self.items = []
        self.text = text
        self.title = title
        self.main_texture = "dialog"
        self.area = [renderer.WIDTH*0.1,renderer.HEIGHT*0.3,renderer.WIDTH*0.9,renderer.HEIGHT*0.95]
        ok = DialogButton("OK", [renderer.WIDTH/2.0,renderer.HEIGHT*0.4],"OK")
        ok.action = self.ok_func
        self.items.append (ok)
        self.down_item = None
        
    def render(self):
        renderer.disable_lighting ()
        renderer.start_pixel_map_mode()
        renderer.set_color([0, 0, 0,.7])
        rendering_utils.draw_rectangle(*self.area)
        renderer.set_color([0, 1, 0, .3])
        rendering_utils.stroke_rectangle(self.area[0] + 5, self.area[1] + 5, self.area[2] - 5, self.area[3] - 25)
        renderer.set_color([0, 1, 0, .7])
        rendering_utils.draw_text([self.area[0] + 15, self.area[3] - 17],  self.title)
        rendering_utils.draw_text([self.area[0] + 15, self.area[3] - 45, self.area[2] - 15, self.area[1] - 30],  self.text)
        for i in self.items:
            i.render()
        renderer.end_pixel_map_mode()
        renderer.enable_lighting ()
     
class DialogButton(MenuItem):
    def render(self):
        renderer.set_color ([1, 1, .4, 1])
        width = len(self.tex) * 8
        #rendering_utils.stroke_rectangle(self.pos[0] - width / 2 - 10, self.pos[1] + 14, self.pos[0] + width / 2 + 10, self.pos[1] - 14)
        rendering_utils.draw_text([self.pos[0] - width / 2, self.pos[1]], self.tex)



        
class MainMenu(Menu):
    def __init__ (self,game):
        Menu.__init__(self)
        self.game = game
        self.main_texture = "mainmenu_new"
        self.area = [0,0,renderer.WIDTH,renderer.HEIGHT]
        start_game = MenuItem ("Start Game",[renderer.WIDTH/2.0,renderer.HEIGHT*0.5],"startgame")
        start_game.action = self.start_game
        self.items.append (start_game)

        story = MenuItem ("Story",[renderer.WIDTH/2.0,renderer.HEIGHT*0.4],"story")
        story.action = self.story
        self.items.append (story)
        
        tutorial = MenuItem ("Tutorial",[renderer.WIDTH/2.0,renderer.HEIGHT*0.3],"tutorial")
        tutorial.action = self.tutorial
        self.items.append (tutorial)
        
        #only display resume if there is a game to resume
        if self.game.playing:
            resume = MenuItem ("Resume",[renderer.WIDTH/2.0,renderer.HEIGHT*0.2],"resume")
	    resume.action = self.resume
	    self.items.append (resume)
	
        quit = MenuItem ("Quit",[renderer.WIDTH/2.0,renderer.HEIGHT*0.1],"quit")
        quit.action = self.quit
        self.items.append (quit)
        
        
        #print self.items
        
    def start_game (self):
        #print "start game"
        self.game.menu = 0
        self.game.playing = True
        
        self.game.currentLevel = 0
        self.game.start_game ()
    
    def tutorial(self):
        #print "tutorials"
        self.game.menu = 0
        self.game.playing = True
        self.game.currentLevel = 0
        self.game.start_tutorial ()

    def story(self):
        #print "story"
        self.game.menu = 0
        self.game.playing = True
        self.game.currentLevel = 0
        self.game.start_story()
    
    
    def resume(self):
        self.game.menu = 0
        
    
    
    def quit(self):
        sys.exit()
    
        
  
