"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
import pygame, ctypes
from pygame.locals import *
from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
import texture_manager
import math


WIDTH = 640
HEIGHT = 480
VIEW_PORT_HEIGHT = 400
fAspect = WIDTH/float(HEIGHT)

FLOAT_ARRAY = ctypes.c_float * 4
camera_pos = [5,5,10]
camera_angle = 0
view_angle = 45
surface = 0
main_font =0

max_height = 100
max_sideways = 1
min_sideways = -1
min_height = 1

def start_pixel_map_mode ():
    """This allows to easily write to pixels rather than world coordinates"""
    glMatrixMode(GL_MODELVIEW)
    glPushMatrix()
    glLoadIdentity ()
    glMatrixMode(GL_PROJECTION)
    glPushMatrix()
    glLoadIdentity ()
    glOrtho (0,WIDTH,0,HEIGHT,1,-1)
    glDisable (GL_DEPTH_TEST)
    glDisable (GL_LIGHTING)
    

def end_pixel_map_mode ():
    glMatrixMode(GL_PROJECTION)
    glPopMatrix()
    glMatrixMode(GL_MODELVIEW)
    glPopMatrix()
    glEnable (GL_DEPTH_TEST)
    glEnable (GL_LIGHTING)

def start_simple_perspective ():
    glDisable(GL_LIGHTING)
    glMatrixMode(GL_MODELVIEW)
    glPushMatrix()
    glLoadIdentity ()
    glMatrixMode(GL_PROJECTION)
    glPushMatrix()
    glLoadIdentity ()
    gluPerspective(view_angle,fAspect,0.1, 10000.0)
    glTranslatef (0,0,-1)
    glMatrixMode(GL_MODELVIEW)
    glDisable (GL_TEXTURE_2D)
def end_simple_perspective ():
    glEnable(GL_LIGHTING)
    glMatrixMode(GL_PROJECTION)
    glPopMatrix()
    glMatrixMode(GL_MODELVIEW)
    glPopMatrix()
    glEnable (GL_TEXTURE_2D)
    
    
def set_model_view_matrix ():
    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    gluPerspective(45.0, fAspect, 0.1, 10000.0)
    glRotatef (view_angle,-1,0,0)
    glRotatef (camera_angle,0,0,1)
    glTranslatef (*[-a for a in camera_pos])
    glMatrixMode(GL_MODELVIEW)


def resize():
    glViewport(0, 0, WIDTH, HEIGHT)
    set_model_view_matrix ()
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()


def set_camera_pos (p):
    global camera_pos
    if (p[2]>max_height):
        return
    if (p[2]<min_height):
        return # dont fo sliding along the floor
    for i in range(2):
        if (p[i]>max_sideways):
            p[i] = max_sideways
        if (p[i]<min_sideways):
            p[i] = min_sideways
        
    camera_pos = p
    set_model_view_matrix()


def get_camera_pos ():
    return camera_pos    
def set_camera_angle (a):
    global camera_angle
    camera_angle = a
    set_model_view_matrix()
def get_camera_angle ():
    return camera_angle
def camera_distance_change(delta):
    
    camera_pos[2] += delta
    if (camera_pos[2] < 1):
        camera_pos[2] = 1
    elif (camera_pos[2] > 12):
        camera_pos[2] = 12
    set_model_view_matrix()
    
def init_fonts():
    global main_font 
    pygame.font.init ()
    main_font = font = pygame.font.Font(None, 36)
    
def init(size,fullscreen):
    global surface 
    global WIDTH 
    global HEIGHT 
    global VIEW_PORT_HEIGHT 
    if (fullscreen):
        video_flags = OPENGL|DOUBLEBUF|HWSURFACE|FULLSCREEN
    else:
        video_flags = OPENGL|DOUBLEBUF
    init_fonts()
    WIDTH, HEIGHT = size
    VIEW_PORT_HEIGHT = HEIGHT - 80
    
    pygame.init()
    surface = pygame.display.set_mode((WIDTH, HEIGHT), video_flags)
    #print surface.get_size()
    pygame.display.set_caption('Robots')
    resize()
    glutInit(sys.argv, len(sys.argv))
    # Enable opengl options
    glEnable(GL_CULL_FACE)
    glFrontFace(GL_CCW)
    glClearColor(0.0, 0.0, 0.0, 0.0)
    glShadeModel(GL_SMOOTH)
    
    glEnable(GL_LIGHTING)
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, FLOAT_ARRAY(0.05, 0.05, 0.05, 1.0))
    #Light 0
    glEnable(GL_LIGHT0)
    glLightfv(GL_LIGHT0, GL_AMBIENT, FLOAT_ARRAY(0.3,0.3,0.5, 1.0))
    glLightfv(GL_LIGHT0, GL_DIFFUSE, FLOAT_ARRAY(1,1.0,0.7, 1.0))
    glLight (GL_LIGHT0,GL_POSITION,[1,-0.0,0.5,0])
    #Light 1
    glEnable(GL_LIGHT1)
    glLightfv(GL_LIGHT1, GL_AMBIENT, FLOAT_ARRAY(0,0,0, 1.0))
    glLight (GL_LIGHT1,GL_DIFFUSE,[0.3,0.3,0])
    glLight (GL_LIGHT1,GL_POSITION,[1,1,3,1]) #This doesnt belong here - but for now
    glEnable (GL_POLYGON_SMOOTH);
    glEnable (GL_BLEND);
    #glEnable (GL_MULTISAMPLE);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glHint (GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
    glPointSize(1.0)
    glEnable(GL_TEXTURE_2D)
    glEnable(GL_COLOR_MATERIAL)
    glEnable(GL_DEPTH_TEST)
    glClearDepth(1.0)
    glDepthFunc(GL_LEQUAL)
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST)

def clear_depth ():
    clearOptions = GL_DEPTH_BUFFER_BIT
    glClear(clearOptions)

def render():
    clearOptions = GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT
    glClear(clearOptions)
   
    # Do more drawing This is where the level loading should be.
    glColor3f(1.0, 1.0, 1.0)
    
def get_world_pos (scr_pos, z_plane = 0):
    """ gets the world pos for the z=0 plane"""
    
    point = gluUnProject(float(scr_pos[0]),float ((HEIGHT-1)-scr_pos[1]),0.99)
    f = float (camera_pos[2]-z_plane)/(camera_pos[2]-point[2])
    point_on_plane = [a + f*(b-a) for a,b in zip (camera_pos,point)]
    
    return point_on_plane
    
def bind_texture (name):
    glBindTexture(GL_TEXTURE_2D,texture_manager.getTexture(name))
def bind_texture_obj (tex):
    glBindTexture(GL_TEXTURE_2D,tex)

def set_color(c):
    if len(c) == 4:
        glColor4f(*c)
    else:
        glColor3f(*c)

def set_blend_color(c, blend):
    r, g, b = c
    glColor4f(r, g, b, blend)

def push_position():
    glPushMatrix ()
def pop_position():
    glPopMatrix ()
def translate(x,y,z):
    glTranslatef (x,y,z)
def rotate(angle,x,y,z):
    glRotatef (angle,x,y,z)
    
def disable_lighting():
    glDisable (GL_LIGHTING)
def enable_lighting():
    glEnable (GL_LIGHTING)

def enable_blending():
    glBlendFunc(GL_ONE, GL_ONE)

def disable_blending():
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) 
