"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
from engine import Engine
    
from game_objects import Enemy, MemoryBank
from level import Level
from tile_objects import  Wall, Gate, Switch ,TokenLocation, Spawn
import renderer
import dragger
import texture_manager
import menus
import stars
#import sound_manager
from game_objects import getCommandFromSizeIndex
CONSTANTS = """
WIDTH
HEIGHT    
""".strip().split()
globs = globals()
for i in CONSTANTS:
    globs[i] = i

import rendering_utils, level_utils
from OpenGL.GLUT import *
from constants import *

available_tokens_pos = [0,0]
token_image_width = 40
token_image_height = 40
robot_tokens_pos = [0,token_image_height]

num_stars = 200

spawn_button_pos = [465,50]
spawn_button_diameter = 40
fairy_doom_pos = [595,50]
fairy_doom_diameter = 40
slot_width = 42

COMMANDS_TO_TEXTURE = {
    'F1': 'forward1',
    'F5': 'forward5',
    'L': 'left',
    'R': 'right',
    'S': 'seek',
    'H': 'hack2',
    'P': 'push',
} 



class Robots(Engine):
    
    def init_stars (self):
        self.stars = []
        for i in range (num_stars):
            self.stars.append (stars.star())
        
    def __init__(self, renderer, size, fullscreen):
        
        Engine.__init__(self, renderer,size,fullscreen)
        textureList = {}
        for fname in os.listdir("data"):
            if fname.endswith(".jpg") or fname.endswith(".png"):
                fstub = fname[:-4]#4 is .png/jpg
                if (not fstub in textureList):
                    textureList[fstub]= fname
            
        texture_manager.loadTextures (textureList)
        #texture_manager.loadTextures (["metal2","floor1","mur040", "panel","spawn","switch","Forward2Token","mainmenu","start_game"])
        #sound_manager.loadSounds (["gate_move"])
        self.in_menu = True  
        self.tutorial = False
        self.level = 0
        self.init_world()
        self.menu = menus.MainMenu (self)
        self.init_stars ()
        self.dragger = dragger.dragger()
        self.dragger.set_world_pos_getter (renderer.get_world_pos)
        self.dragger.set_camera_funcs (renderer.get_camera_pos ,renderer.set_camera_pos , renderer.get_camera_angle,renderer.set_camera_angle,renderer.camera_distance_change )
        self.dragger.set_screen_size ([renderer.WIDTH,renderer.HEIGHT])
       
        
    def hit_button (self,button_pos, diameter, mouse_pos):
        pos = [mouse_pos[0],mouse_pos[1]]
        pos[1] = (self.renderer.HEIGHT - 1)- mouse_pos[1]
        #print rendering_utils.subtract (mouse_pos,button_pos)
        #print rendering_utils.length2 ( rendering_utils.subtract (pos,button_pos))
        return (rendering_utils.length2 ( rendering_utils.subtract (pos,button_pos)) < diameter)
        
    def load_level (self, name):
        #print "load_level: %s" % name
        self.level = level_utils.load_level(self, name)
        
        self.renderer.max_sideways = max (self.level.width,self.level.height) + 25
        self.renderer.min_sideways = - 25
        self.renderer.max_height = 200
        
    def init_world(self):
        self.selected_spawn = None
        self.new_robot = None
        self.down_spawn = None
        if (self.level):
            self.dragger.set_camera_angle_func(0)
            self.dragger.centre_on_square ([renderer.WIDTH/2,renderer.HEIGHT/2],[(self.level.width-1)/2.0,(self.level.height-1)/2.0])
    def star_front (self):
        return self.total_time / 1000.0
    def execute(self):
        if (self.level and not self.menu):
            self.level.execute()
            if (self.level.complete):
                self.complete_level()
                
        #Move stars
        for i in range (10):
            self.stars.pop()
            self.stars.insert (0,stars.star(-self.star_front()-8))
    def render_cool_box (self,x1,y1,x2,y2):
        renderer.set_color([0, 0, 0,.7])
        rendering_utils.draw_rectangle(x1, y1, x2, y2)
        renderer.set_color([0, 1, 0, .3])
        rendering_utils.stroke_rectangle(x1,y1, x2, y2)
        
    def render_token_list (self,pos,commands,length):
        current_pos = pos [:]
        renderer.set_color ([1,1,1])
        #renderer.bind_texture ("metal2")
        #rendering_utils.renderQuad (pos[0],pos[1],token_image_width*length,token_image_height+pos[1],texturePacking=(10,1))
        
        #renderer.set_color ([0,0,0])
        
        
        if (length > 0):
            self.render_cool_box(pos[0], pos[1], pos[0] +slot_width * length, pos[1] +40)
            if (len (commands) == 0 and length > 4):
                renderer.set_color ([0.7,0.7,0.7])
                rendering_utils.draw_text([pos[0], pos[1]+30, slot_width * length, 40], "Choose commands from below for this robot's program.", font = GLUT_BITMAP_8_BY_13)
            
        for command in commands:
            s = command.to_string()
            #rendering_utils.draw_text (current_pos,s,font = GLUT_BITMAP_8_BY_13)
            l = token_image_width*command.length
            prior_blank =  (slot_width*(command.length))/2.0-token_image_width/2.0
            token_pos = [0,0]
            token_pos [0] = current_pos[0] + prior_blank
            token_pos [1] = current_pos[1]
            renderer.bind_texture(COMMANDS_TO_TEXTURE.get(s, 'forward5'))
            renderer.set_color([0, 0, 0,.7])
            box_area = [0,0,20,20]
            rendering_utils.draw_rectangle(token_pos[0]-prior_blank, token_pos[1], token_pos[0] +40+prior_blank, token_pos[1] +40)
            renderer.set_color([0, 1, 0, .3])
            rendering_utils.stroke_rectangle(token_pos[0]-prior_blank, token_pos[1], token_pos[0] +40+prior_blank, token_pos[1] +40)
            renderer.set_color([1,1,1])
            rendering_utils.renderQuad(
                token_pos[0],
                token_pos[1]+1,
                token_pos[0] + token_image_width,
                token_pos[1] + token_image_width,
                texturePacking=(1, 1)
            )
            #renderer.set_color([0, 1, 0, .7])
            #rendering_utils.draw_text([box_area[0] + 15, box_area[3] - 17],  self.title)
            #rendering_utils.draw_text([box_area[0] + 15, box_area[3] - 45, box_area[2] - 15, box_area[1] - 30],  self.text)
            current_pos[0] += slot_width*command.length
            
        
    def render_console (self):
        """UI rendering"""
        renderer.start_pixel_map_mode()
        console_width = renderer.WIDTH
        console_height = renderer.HEIGHT - renderer.VIEW_PORT_HEIGHT
       
        renderer.set_color([1,1,1]) 
        renderer.bind_texture ("metal2")
        rendering_utils.renderQuad (0,0,console_width,console_height,texturePacking=(10,1))
        

        if self.tutorial:
            renderer.set_color ([1,1,1])
            rendering_utils.draw_text([10, 460, 320, 0], self.tutorial_text, font = GLUT_BITMAP_8_BY_13)
                    
        renderer.set_color ([1,1,1])
        #1rendering_utils.draw_text([420,10], "Click", font = GLUT_BITMAP_8_BY_13)
        renderer.bind_texture("spawn2")
        rendering_utils.renderQuad(spawn_button_pos[0]-spawn_button_diameter, spawn_button_pos[1]-spawn_button_diameter, spawn_button_pos[0]+spawn_button_diameter, spawn_button_pos[1]+spawn_button_diameter, texturePacking=(1, 1))
        renderer.bind_texture("fairybomb-doomsbringer")
        rendering_utils.renderQuad(fairy_doom_pos[0]-fairy_doom_diameter, fairy_doom_pos[1]-fairy_doom_diameter, fairy_doom_pos[0]+fairy_doom_diameter, fairy_doom_pos[1]+fairy_doom_diameter, texturePacking=(1, 1))
        renderer.set_color ([1,1,1])
        renderer.bind_texture("num_"+str(self.level.remaining_robots))
        rendering_utils.renderQuad(495, 10, 555, 70, texturePacking=(1, 1))
        
        #rendering_utils.draw_text ([460, 10, 500, 70],str(self.level.remaining_robots), font = GLUT_BITMAP_TIMES_ROMAN_24)
        available_tokens = self.level.available_tokens
        
        self.render_token_list (available_tokens_pos,available_tokens, self.level.available_token_size)

        if (self.selected_spawn):
            self.render_token_list (robot_tokens_pos,self.new_robot.commands.memory, self.new_robot.commands.maxLength)

        renderer.end_pixel_map_mode()
        
    def render_stars (self):
        renderer.set_color ([0,0,0])
        renderer.start_simple_perspective()
        renderer.translate (0,0,self.star_front())
        change_point = num_stars/10
        next_change_point = change_point
        c = 0.0
        i = 0
        for star in self.stars:
            if (i > next_change_point):
                
                next_change_point += change_point
                c += 0.3
                if c > 1:
                    c = 1.0
                renderer.set_color ([c,c,c])
            i += 1
            
            rendering_utils.draw_dot (star.pos)
        renderer.end_simple_perspective()
        
    def render(self,ticks,phase):
        self.render_stars ()
        renderer.clear_depth()
        if (self.level):
            self.level.render(ticks,phase)
            self.render_console()
        if (self.menu):
            self.menu.render()
    
    def make_new_robot(self):
        self.new_robot = self.level.add_robot(self.selected_spawn.pos)
        self.new_robot.created = False
        self.new_robot.dir = self.selected_spawn.dir
        self.new_robot.commands = MemoryBank(MEMORY_SIZE)
        self.new_robot.spawn = self.selected_spawn
        self.level.launch_robot (self.new_robot)
    
    def launch_robot (self):
        if (self.selected_spawn):
            #needs to be moved back to the spawn point after, the user has seen where itll move to
            self.level.remaining_robots -= 1
            self.new_robot.moveToSpawn()

            self.new_robot.created = True

            
            #then create new one
            if (self.level.remaining_robots <= 0):
                self.selected_spawn = None
            else:
                self.make_new_robot()
                    
            
            
    def mouse_down (self,pos,buttons):
        if (self.menu):
            self.menu.mouse_down(pos,buttons)
        else:
            if (pos[1] > renderer.VIEW_PORT_HEIGHT):
                if self.tutorial:
                    if (self.hit_button (spawn_button_pos,spawn_button_diameter,pos)):
                        self.level.complete = True
                else:
                    if self.hit_button (spawn_button_pos,spawn_button_diameter,pos): 
                        if (self.selected_spawn and self.new_robot.commands.length>0):
                            self.launch_robot ()
                        else:
                            pass
                    elif self.hit_button (fairy_doom_pos,fairy_doom_diameter,pos):
                        self.selected_spawn = None
                        self.new_robot = None
                        self.resetLevel()
                        self.down_spawn = None
                
                    elif (self.selected_spawn and pos[1] > renderer.HEIGHT - token_image_height and pos [0] < self.level.available_token_size * slot_width):
                        new_command = getCommandFromSizeIndex (self.level.available_tokens,pos [0]/slot_width).clone()
                        new_command.set_robot(self.new_robot)
                                        
                        self.new_robot.commands.addCommand(new_command)
                    
                    elif (self.selected_spawn and pos[1] > renderer.HEIGHT - token_image_height*2 and pos[1] < renderer.HEIGHT - token_image_height and pos[0] < self.new_robot.commands.maxLength*slot_width):
                        self.new_robot.commands.deleteCommand(pos[0]/slot_width)
                    
                        self.new_robot.moveToSpawn()
            else:
                
                worldpos = self.dragger.world_pos_getter(pos)
                if not self.tutorial:
                    for spawn in self.level.spawns:
                        if self.level.remaining_robots >= spawn.activated and spawn.is_inside(worldpos):
                            self.down_spawn = spawn              
                            break

                self.dragger.mouse_down (pos,buttons)
                self.moved = False

                    
                    

    def mouse_up (self,pos):
        if (self.menu):
            self.menu.mouse_up(pos)
        else:
            worldpos = self.dragger.world_pos_getter(pos)
            nospawn = True
            if self.down_spawn:
                if self.down_spawn.is_inside(worldpos) \
                        and self.down_spawn.activated:
                    if (self.selected_spawn):
                        self.selected_spawn.selected = False
                        self.level.remove_robot(self.new_robot)

                    self.selected_spawn = self.down_spawn
                    self.down_spawn = None

                    self.selected_spawn.selected = True
                    nospawn = False

                    self.make_new_robot()

                    pass
                else:
                    self.down_spawn = None
            
            if not self.moved:
                if self.selected_spawn and nospawn:
                    self.selected_spawn.selected = False
		    self.level.remove_robot(self.new_robot)
		    self.selected_spawn = None
            
        self.dragger.mouse_up(pos)

    def mouse_move (self,pos):
        if (self.menu):
            self.menu.mouse_move(pos)
        else:
            self.dragger.mouse_move (pos)
            self.moved = True
    def wheel_up (self):
        if (not self.menu):
            self.dragger.wheel_up ()
    def wheel_down (self):
        if (not self.menu):
            self.dragger.wheel_down ()
            
            
    def complete_level (self):
        self.running = False
        
        if self.tutorial:
            self.next_level()
        else:
            self.menu = menus.OkDialog  (self.level.title,self.level.text,self.next_level)
    
    def start_game (self):
        max_level = level_utils.load_config(self, "level.cfg")
        self.max_level = max_level
        
        self.tutorial = False
        self.next_level()
        
    def start_tutorial(self):
        max_level = level_utils.load_config(self, "tutorial.cfg")
        self.max_level = max_level
        
        self.tutorial = True
        self.next_level()

    def start_story(self):
        max_level = level_utils.load_config(self, "story.cfg")
        self.max_level = max_level
        
        self.tutorial = True
        self.next_level()
        
    def next_level (self):
        if not Engine.nextLevel(self):
            self.level = 0
            
            if self.tutorial:
                self.in_menu = True
                self.playing = False
                self.menu = menus.MainMenu (self)
                
                return
        
        #print "loaded level"
            
        self.init_world()
        self.menu = 0
        self.running = True
        if self.tutorial:
            self.level.doTutorialStep()
        
        

def main(pos, fullscreen = False):
    r = Robots(renderer,pos,fullscreen)

    r.start()
