"""
 This file is part of Space Robodyssey.

    Space Robodyssey is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Space Robodyssey is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Space Robodyssey.  If not, see <http://www.gnu.org/licenses/>.
"""
class Command:
    robot = 0
    def __init__(self):
        self.isValid = True
        self.length = 1

    def set_robot (self,robot):
        self.robot = robot
    def reset(self):
        pass

class ForwardCommand (Command):
    count = 0
    def __init__ (self,count):
        Command.__init__(self)
        self.count = count
        self.origCount = count
        self.hit_something = False
        if count == 1:
            self.length = 1
        elif count == 5:
            self.length = 2
    def execute (self, level):
        if (self.count > 0):
            newPos = [p+ d for p,d in zip(self.robot.get_pos(),self.robot.get_dir())]
            if not level.getMapElement(newPos).habitable():
                self.hit_something = True
            else:
                self.robot.set_pos (newPos)
                self.robot.moving=True
            self.robot.legs_moving = True
        self.count -= 1
        return self.count <= 0
    def to_string(self):
        return "F"+str(self.origCount)

    def clone(self):
        new = ForwardCommand(self.origCount)
        new.robot = self.robot
        return new
    def reset(self):
        self.count = self.origCount
        self.hit_something = False
class SeekCommand (ForwardCommand):
    def __init__(self):
        Command.__init__(self)
        ForwardCommand.__init__(self,1)
        self.length = 2

    def execute (self, level):
        ForwardCommand.execute(self,level)
        self.count = 1
        return (self.hit_something or self.robot.get_map_element().is_interesting())
            
    def to_string(self):
        return "S"
    def clone(self):
        new = SeekCommand()
        new.robot = self.robot
        return new
    def reset(self):
        ForwardCommand.reset(self)

        

        
        

class LeftCommand (Command):
    def __init__(self):
        Command.__init__(self)
    def execute (self, level):
        current_dir = self.robot.get_dir()
        self.robot.set_dir ([-current_dir[1],current_dir[0]])
        return True
    def to_string(self):
        return "L"
    def clone(self):
        new = LeftCommand()
        new.robot = self.robot
        return new

class RightCommand (Command):
    def __init__(self):
        Command.__init__(self)
        self.length = 1
    def execute (self, level):
        current_dir = self.robot.get_dir()
        self.robot.set_dir ([current_dir[1],-current_dir[0]])
        return True
    def to_string(self):
         return "R"
    def clone(self):
        new = RightCommand()
        new.robot = self.robot
        return new
        
class PushCommand (Command):
    def __init__(self):
        Command.__init__(self)
        self.length = 1
    def execute (self, level):
        self.robot.push()
        return True
    def to_string(self):
        
        return "P"
    def clone(self):
        new = PushCommand()
        new.robot = self.robot
        return new
        
class HackCommand (Command): #Hack a switch so the gates stay open
    def __init__(self):
        Command.__init__(self)
        self.length = 4
    def execute (self, level):
        self.robot.hack()
        return True
    def to_string(self):
        return "H"
    def clone(self):
        new = HackCommand()
        new.robot = self.robot
        return new

class RepeatCommand(Command):
    def __init(self):
        Command.__init__(self)
    def execute(self, level):
        self.robot.current_command = 0
        for i in self.robot.commands:
            i.reset()
    def to_string(self):
        return "-"
    def clone(self):
        new = RepeatCommand()
        new.robot = self.robot
        return new

def char_to_command (char):
    if (char == '5'):
        return ForwardCommand (5)
    elif (char == '1'):
        return ForwardCommand (1)
    elif (char == 'l'):
        return LeftCommand ()
    elif (char == 'r'):
        return RightCommand()
    elif char == '-':
        return RepeatCommand()
    elif char == "p":
        return PushCommand()
    elif char == "s":
        return SeekCommand()
    else :
        return ForwardCommand (0) #use this as an empty default

